/*
 * Copyright (c) 2007-2008,2010,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Wed Feb 14 2007
 */
#ifndef H_VIAM_SHARED
#define H_VIAM_SHARED

#include <stdlib.h>

#include "viam/viamtypes.h"

#ifdef VIAM_OPENCV_SUPPORT
# ifndef CV_INLINE
#  define CV_INLINE	static inline
# endif
# define CV_NO_BACKWARD_COMPATIBILITY
# include "opencv2/core/core_c.h"
# ifdef VIAM_DISPLAY
#  include "opencv2/highgui/highgui_c.h"
# endif
#else
# ifdef VIAM_DISPLAY
#  warning "Display facility require opencv which has been disabled in viamlibs."
#  warning "This module will be compiled *without* display."
#  undef VIAM_DISPLAY
# endif
#endif


/* --- global data --------------------------------------------------- */

extern viam_handle_t		vh;		/**< global viam handle */
extern const POM_SENSOR_POS	viam_poszero;	/**< zero position */


/* --- viamlib ------------------------------------------------------- */

#define VIAM_RETURN_IFRUNNING(r, a)					\
  do if (SDI_F->init_done) {						\
    *(r) = S_viam_MODULE_RUNNING; return (a);				\
  } while(0)

#define VIAM_MAPERRNO(r)						\
  do switch(viam_errno(vh)) {						\
      _viam_map_err(r, Eviamlib_OK,		S_viam_UNKNOWN_ERROR);	\
      _viam_map_err(r, Eviamlib_MEMORY,		S_viam_MEMORY);		\
      _viam_map_err(r, Eviamlib_OPENCV,		S_viam_OPENCV);		\
      _viam_map_err(r, Eviamlib_FWBUS,		S_viam_FWBUS);		\
      _viam_map_err(r, Eviamlib_FILE_IO,	S_viam_FILE_IO);	\
      _viam_map_err(r, Eviamlib_NO_CAMERA,	S_viam_NO_CAMERA);	\
      _viam_map_err(r, Eviamlib_NOT_IMPLEMENTED,S_viam_NOT_IMPLEMENTED);\
      _viam_map_err(r, Eviamlib_DUPLICATE_ID,	S_viam_DUPLICATE_ID);	\
      _viam_map_err(r, Eviamlib_BAD_ID,		S_viam_BAD_ID);		\
      _viam_map_err(r, Eviamlib_NO_SUCH_DEVICE,	S_viam_NO_SUCH_DEVICE);	\
      _viam_map_err(r, Eviamlib_NO_SUCH_IMAGE,	S_viam_NO_SUCH_IMAGE);	\
      _viam_map_err(r, Eviamlib_NO_SUCH_CAMERA,	S_viam_NO_SUCH_CAMERA);	\
      _viam_map_err(r, Eviamlib_NO_SUCH_BANK,	S_viam_NO_SUCH_BANK);	\
      _viam_map_err(r, Eviamlib_NO_SUCH_FILTER,	S_viam_NO_SUCH_FILTER);	\
      _viam_map_err(r, Eviamlib_NOT_CONFIGURED,	S_viam_NOT_CONFIGURED);	\
      _viam_map_err(r, Eviamlib_CONFIGURED,	S_viam_CONFIGURED);	\
      _viam_map_err(r, Eviamlib_UNSUPPORTED_HWMODE, S_viam_UNSUPPORTED_HWMODE);	\
      _viam_map_err(r, Eviamlib_INVALID_VALUE,	S_viam_INVALID_VALUE);	\
      _viam_map_err(r, Eviamlib_IMAGE_GEOMETRY,	S_viam_IMAGE_GEOMETRY);	\
      _viam_map_err(r, Eviamlib_BAD_IMAGE_COUNT,S_viam_BAD_IMAGE_COUNT);\
      _viam_map_err(r, Eviamlib_CALIBRATING,	S_viam_CALIBRATING);	\
      _viam_map_err(r, Eviamlib_NOT_CALIBRATED,	S_viam_NOT_CALIBRATED);	\
  } while(0)
#define VIAM_RETURN_WITHERRNO(r, a)					\
  do {									\
    VIAM_MAPERRNO(r);							\
    return (a);								\
  } while(0)
#define _viam_map_err(r, f, t) case (f): *(r) = (t); break


/* --- banks --------------------------------------------------------- */

typedef struct viam_posterdata_t viam_posterdata_t;

/** module internal data attached to each camera of a bank */
typedef struct viam_imagedata_t {
#ifdef VIAM_DISPLAY
  char *		window;		/**< window name for display or NULL */
  unsigned int		wwidth, wheight;/**< window size */
  viam_onoff_t		vtags;		/**< display performance tags */
  struct timeval	ptacq;		/**< previous time of acquisition */
  double		fps;		/**< current fps */
#endif
  char *		file;		/**< file name for save or NULL */
  unsigned long		seq;		/**< image sequence number */

  unsigned long		pomSeq;		/**< -1U for live pom or file seq # */
  POSTER_ID		pomFrameId;	/**< frame poster */
  POM_SENSOR_POS 	pomFrame;	/**< frame position */
  struct timeval	tacq;		/**< time of frame acquisition */
} viam_imagedata_t;

/** module internal data attached to each bank */
typedef struct viam_bankdata_t {
  ViamImageUpdate	buffers;	/**< buffering type */
  ViamActivate		tags;		/**< pom tagging */
  viam_posterdata_t *	pdata;		/**< poster private data */
  viam_imagedata_t *	idata;		/**< image specific data */
  int			nidata;		/**< size of above array */
} viam_bankdata_t;

static inline viam_imagedata_t *
viamIData(viam_bank_t bank, unsigned int image)
{
  viam_bankdata_t *d = viam_bank_usrdata(bank);
  if (!d) return NULL;

  if (d->nidata <= image) {
    viam_imagedata_t *c;
    c = realloc(d->idata, (image+1)*sizeof(*d->idata));
    if (!c) return NULL;
    d->nidata = image+1;
    d->idata = c;

#ifdef VIAM_DISPLAY
    d->idata[image].window = NULL;
    d->idata[image].wwidth = d->idata[image].wwidth = 0;
    d->idata[image].vtags = VIAM_OFF;
    d->idata[image].ptacq = (struct timeval){ 0, 0};
    d->idata[image].fps = 0.;
#endif
    d->idata[image].file = NULL;
    d->idata[image].seq = 0;
    d->idata[image].pomFrameId = NULL;
    d->idata[image].pomFrame = viam_poszero;
    d->idata[image].tacq = (struct timeval){ 0, 0};
  }
  return &d->idata[image];
}


/* --- private functions --------------------------------------------- */

STATUS			viamAcqInit(int *report);
STATUS			viamAcqEnd(void);

STATUS			viamPomInit(viam_bank_t bank, int *report);
int			viamPomGetTags(viam_bank_t bank);

#ifdef VIAM_DISPLAY
STATUS			viamDisplayInit(int *report);
void			viamDisplaySignal(const viam_bank_t bank);
#endif /* VIAM_DISPLAY */

viam_posterdata_t *	viamPosterCreate(viam_handle_t vh, viam_bank_t bank,
				int buffers, int *report);
STATUS			viamPosterDelete(viam_handle_t vh,
				viam_posterdata_t *pdata);
STATUS			viamPosterAcqLock(viam_handle_t vh,
				viam_posterdata_t *pdata, int *report);
STATUS			viamPosterAcqRelease(viam_handle_t vh,
				viam_bank_t bank, viam_posterdata_t *pdata,
				int *report);
IplImage *		viamPosterCurrentImage(viam_handle_t vh,
				viam_posterdata_t *pdata, int i);
IplImage *		viamPosterNewImage(viam_handle_t vh,
				viam_posterdata_t *pdata, int i);

#endif /* H_VIAM_SHARED */
